;;; CL-Wav-Synth -  Manipulate WAV files
;;;
;;; Copyright (C) 2006 Philippe Brochard (hocwp@free.fr)
;;;
;;; #Date#: Fri May 26 10:49:09 2006
;;;
;;; **********************************************
;;; The authors grant you the rights to distribute
;;; and use this software as governed by the terms
;;; of the Lisp Lesser GNU Public License
;;; (http://opensource.franz.com/preamble.html),
;;; known as the LLGPL.
;;; **********************************************


(in-package :cl-wav-synth)

(defclass next-tutorial-class () ())

(defparameter *hit-delay* 0)



(defparameter *current-def-tuto* 0)
(defparameter *tutorial-state* 0) 


(defparameter *tutorial-hash* (make-hash-table))



(defun write-msg (string &optional (skip-sleep #\_))
  (clim-listener::print-listener-prompt *standard-output* clim:*application-frame*)
  (force-output)
  (loop for c across string
	with do-sleep = t do
	(if (eql c skip-sleep)
	    (setf do-sleep (not do-sleep))
	    (princ c))
	(force-output)
	(when do-sleep
	  (sleep *hit-delay*)))
  (terpri))


(defmacro define-tutorial (form1 comment &body form2)
  "Define a tutorial: if form1 is a string, then print form1 (with delay)
and evaluate form2. Else print form1 and evaluate form1 (form2 is ignored)"
  `(progn
    (setf (gethash *current-def-tuto* *tutorial-hash*)
     (list ',form1
      ,comment
      #'(lambda ()
	  ,@(if (stringp form1)
		`((write-msg ,form1)
		  (format t "  => ~A~%" ,comment)
		  (force-output)
		  ,@form2)
		`((write-msg (format nil "~(~S~)" ',form1))
		  (format t "  => ~A~%" ,comment)
		  (force-output)
		  ,form1))
	  (force-output))))
     (incf *current-def-tuto*)))

    

(define-tutorial "Load As Sample _(pathname)_ synth.wav"
    "Load a sample in the editor. Notice that you have the auto completion
with Tab or Space key."
  (wav-clim::com-load-as-sample "synth.wav"))

(define-tutorial "Play"
    "Play the current sample"
  (wav-clim::com-play))

(define-tutorial "Load As Sample _(pathname)_ bomb.wav"
    "Load a sample in the editor"    
  (wav-clim::com-load-as-sample "bomb.wav"))

(define-tutorial "Play"
    "Play the current sample"
  (wav-clim::com-play))

(define-tutorial "Set Length _(length)_ 100"
    "Set the display sample length"
  (wav-clim::com-set-length 100))

(define-tutorial "Set Index _(index)_ 10000"
    "Set the display sample index"
  (wav-clim::com-set-index 10000))

(define-tutorial "Reset Bound Values"
    "Adapt the display to the length of current sample"
  (wav-clim::com-reset-bound-values))

(define-tutorial (help 'set-sample)
    "Show the help for the function set-sample")

(define-tutorial (set-sample (null-sample 0.01))
    "Set the current sample to a null sample of 0.01 seconds length")

(define-tutorial "_Reset Bound Values_"
    "Adapt the display to the length of current sample"
  (wav-clim::com-reset-bound-values))

(define-tutorial (help 'sinus-sample)
    "Show the help for the function sinus-sample")

(define-tutorial (set-sample (sinus-sample 0.01 1000))
    "Set the current sample to a sinus sample of 0.01 seconds length
and a frequency of 1000 Hz")

(define-tutorial "_Reset Bound Values_"
    "Adapt the display to the length of current sample"
  (wav-clim::com-reset-bound-values))

(define-tutorial (set-sample (noise-sample 1 100))
    "Set the current sample to a noise sample of 0.01 seconds length
and a frequency of 100 Hz")

(define-tutorial "_Reset Bound Values_"
    "Adapt the display to the length of current sample"
  (wav-clim::com-reset-bound-values))

(define-tutorial (set-sample (line-sample* 1 200 :ampls '((0 0)
							  (0.1 10000)
							  (0.2 5000)
							  (0.4 30000)
							  (0.5 -5000)
							  (0.6 5000)
							  (0.7 -10000)
							  (0.8 -5000)
							  (0.9 -20000)
							  (1 0))))
    "Set the current sample to a line sample defined with ampls parameters.
For the first parameter of each control point, 0 is the period begining and
1 is the end. The second parameter is the amplitude value")

(define-tutorial "Set Length _(length)_ 200"
    "Set the display sample length"
  (wav-clim::com-set-length 200))

(define-tutorial "Set Amplitude _(amplitude)_ 40000"
    "Set the display sample amplitude"
  (wav-clim::com-set-amplitude 40000))

(define-tutorial "_Reset Bound Values_"
    "Adapt the display to the length of current sample"
  (wav-clim::com-reset-bound-values))

(define-tutorial (set-sample (env it (0 0) (0.1 1)))
    "Set the current sample envelope. First parameter in second
and the second parameter is the amplitude multiplicator")

(define-tutorial (set-sample (env it (0.1 1) (0.2 0.68)))
    "Set the current sample envelope. First parameter in second
and the second parameter is the amplitude multiplicator")

(define-tutorial (set-sample (env-sinus it :start 0.2 :period 3 :min 0.5))
    "Set the current sample envelope with a sinusoidale envelope.")

(define-tutorial (set-sample (env it (0.8 1) (1 0)))
    "Set the current sample envelope.")

(define-tutorial "Play"
    "Play the current sample"
  (wav-clim::com-play))

(define-tutorial (set-sample (mix it (env (noise-sample 1.5 200) (0 0) (0.5 0.1) (1.5 1))))
    "Mix the current sample and a noise sample of 1.5 seconds length and 200 Hz
with a growing amplitude")

(define-tutorial "_Reset Bound Values_"
    "Adapt the display to the length of current sample"
  (wav-clim::com-reset-bound-values))

(define-tutorial "Play"
    "Play the current sample"
  (wav-clim::com-play))
    
(define-tutorial "End of tutorial" "Thanks to have looked at this tutorial" nil)




(define-wav-function tutorial (&optional (hit-delay 0.1))
  "Run a tutorial to demonstrate a use of the sample editor"
  (setf *hit-delay* hit-delay
	*tutorial-state* 0)
  (next-tutorial))

(define-wav-function next-tutorial ()
  "Run the next tutorial step"
  (let ((fun (gethash *tutorial-state* *tutorial-hash*)))
    (if fun
	(progn
	  (funcall (third fun))
	  (incf *tutorial-state*)
	  (clim:present "Next tutorial" 'next-tutorial-class))
	(setf *tutorial-state* 0))))



(defun auto-doc-tutorial-text (filename)
  "automatically create a tutorial documentation of cl-wav-synth in text form"
  (with-open-file (stream filename :direction :output
			  :if-exists :supersede
			  :if-does-not-exist :create)
    (format stream "CL-WAV-SYNTH: Express noises as you think~%~%~%")
    (format stream "This is a copy of the sample tutorial available with
the 'Tutorial' command in cl-wav-synth.~%~%~%")
    (maphash #'(lambda (k v)
		 (declare (ignore k))
		 (clim-listener::print-listener-prompt stream clim:*application-frame*)
		 (if (stringp (first v))
		     (format stream "~A~%" (first v))
		     (format stream "~(~S~)~%" (first v)))
		 (format stream "  => ~A~%~%" (second v)))
	     *tutorial-hash*)
    (format stream "~%~%(This documentation has been automatically generated with the
function auto-doc-tutorial)~%~%")))



(defun subst-in-tree-from-doc (original-tree)
  "Add reference in the cl-wav-synth documentation"
  (let ((string (format nil "~(~S~)" original-tree)))
    (labels ((rec (tree)
	       (if (atom tree)
		   (multiple-value-bind (val foundp)
		       (gethash tree *wav-function-hash*)
		     (declare (ignore val))
		     (when foundp
		       (setf string
			     (subst-string (format nil "<a href=\"documentation.html#~A\">~(~S~)</a>"
						   (escape-name (format nil "~S" tree))
						   tree)
					   (format nil "~(~S~)" tree)
					   string))))
		   (progn
		     (rec (car tree))
		     (when (cdr tree)
		       (rec (cdr tree)))))))
      (rec original-tree)
      string)))



(defun auto-doc-tutorial-html (filename)
    "automatically create a tutorial documentation of cl-wav-synth in html form"
  (with-open-file (stream filename :direction :output
			  :if-exists :supersede
			  :if-does-not-exist :create)
    (format stream "<?xml version=\"1.0\"?>
<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"
    \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">
<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">
 <head>
 <title>cl-wav-synth</title>
 <meta http-equiv=\"Content-Type\" content=\"text/html; charset=ISO-8859-1\"/>
</head>

<body>~%")
    (format stream "<h1><a id=\"top\" name=\"top\">
CL-WAV-SYNTH: Express noises as you think
</a></h1>~%")
    (format stream "<h2>This is a copy of the sample tutorial available with
the 'Tutorial' command in cl-wav-synth.</h2>")
    (maphash #'(lambda (k v)
		 (declare (ignore k))
		 (format stream "<p>")
		 (clim-listener::print-listener-prompt stream clim:*application-frame*)
		 (if (stringp (first v))
		     (format stream "~A~%" (first v))
		     (format stream (subst-in-tree-from-doc (first v))))
		 (format stream "<br/>&nbsp;&nbsp;&nbsp;&nbsp;")
		 (format stream "  => ~A~%~%" (second v))
		 (format stream "</p>"))
	     *tutorial-hash*)
    (format stream "<p><small>(This documentation has been automatically generated with the
function auto-doc-tutorial)</small></p>~%~%")
    (multiple-value-bind (s min h d m y)
	(get-decoded-time)
      (declare (ignore s min h))
      (format stream "<div class=\"footer\">
     <a href=\"mailto:hocwp@free.fr\">Philippe Brochard</a>, ~A ~2,'0D ~2,'0D
   </div>
   <div class=\"check\">
     <a href=\"http://validator.w3.org/check/referer\">

        Valid XHTML 1.0 Strict</a>
  </div>
 </body>
</html>~%" y m d))))


(define-wav-function auto-doc-tutorial (filename &optional (type :text))
  "automatically create a tutorial documentation of cl-wav-synth. Type can be :text or :html"
  (ecase type
    (:text (auto-doc-tutorial-text filename))
    (:html (auto-doc-tutorial-html filename))))





(in-package :cl-wav-synth-clim)

(add-menu-item-to-command-table 'help-command-table "Tutorial" :divider nil)

(define-command (com-tutorial :name t :menu "Run the tutorial"
			      :command-table help-command-table)
    ()
  (cl-wav-synth::tutorial))

(define-command (com-next-tutorial :name t :menu "Next tutorial"
				   :command-table help-command-table)
    ()
  (cl-wav-synth::next-tutorial))

(define-presentation-to-command-translator next-tutorial-transl
    (cl-wav-synth::next-tutorial-class com-next-tutorial help-command-table
				       :gesture :select
				       :documentation "Next tutorial"
				       :pointer-documentation "Next tutorial")
    ()
  ())

(add-command-table-to-listener 'help-command-table)
